
/* Items in the tables are centered by default. Which is ugly
   Left alignment looks better
   Tables on the left too
*/
table.align-default {
   text-align: left;
   margin: 1em 0 1em 0;
}

img.align-default,
.figure.align-default {
   text-align: left;
   margin: 0.5em;
   padding: 0.5em;
}

/* Adjust the tables */

table.table,
table.longtable {
   border: 1px solid #DDD;
   border-collapse: collapse;
}

table.table {
   width: initial;
}

table.longtable {
   width: 100%;
}

table.table td,
table.table th {
   border-left: 1px solid #DDD;
}

tr.row-even td {
   background-color: #F0F6F6;
}


/* pandas dataframe */

table.dataframe {
   margin: 0 0 1em 0;
   color: #000;
   border-collapse: collapse;
}

table.dataframe,
table.dataframe tr,
table.dataframe th,
table.dataframe td {
   border: 1px solid #DDD;
}

table.dataframe th,
table.dataframe td {
   padding: .25em .5em;
}

table.dataframe thead th,
table.dataframe tfoot th {
   color: #333;
   text-align: left;
   background-color: #fff;
}

table.dataframe thead th {
   font-weight: bold;
   font-size: 1em;
   border-bottom: 2px solid #DDD;
}

table.dataframe tbody th,
table.dataframe thead tr th:first-child {
   border-right: 2px solid #DDD;
}

table.dataframe tbody th {
   text-align: right;
}

table.dataframe tfoot th {
   font-weight: normal;
   font-size: 1em;
   border-top: 2px solid #DDD;
}

table.dataframe tbody th,
table.dataframe tbody td,
table.dataframe thead th {
   font-size: .95em;
}

table.dataframe tbody tr:nth-child(odd) {
   background: #F0F6F6;
}

/* Better formatting for numpydoc parameter field lists */

dl.class>dd>dl.field-list,
dl.method>dd>dl.field-list,
dl.function>dd>dl.field-list {
   display: grid;
   grid-template-columns: max-content auto;
   border-left: 5px solid #eee;
}

dl.class>dd>dl.field-list>dt,
dl.method>dd>dl.field-list>dt,
dl.function>dd>dl.field-list>dt {
   grid-column-start: 1;
   padding: 0.25em;
   background-color: rgb(238, 238, 238);
   display: inline-table;
}

dl.class>dd>dl.field-list>dd,
dl.method>dd>dl.field-list>dd,
dl.function>dd>dl.field-list>dd {
   margin-inline-start: 0em;
   grid-column-start: 2;
   padding: 0.25em 0.25em 0 0.25em;
   margin-bottom: 0;
}

dl.class>dd>dl.field-list>dd>dl,
dl.method>dd>dl.field-list>dd>dl,
dl.function>dd>dl.field-list>dd>dl {
   margin-left: 1em;
   margin-bottom: 1em;
}

dl.class>dd>dl.field-list>dd div,
dl.method>dd>dl.field-list>dd div,
dl.function>dd>dl.field-list>dd div{
   margin-top: 0.5em;
}

#sidebar>.nav {
   padding: 0 0 2em 0;
}

#sidebar .nav > li {
  padding: 0.8em 0 0 0;
  display: list-item;
  line-height: 1.1em;
}

.bs-sidenav .nav > .active > a,
.bs-sidenav .nav > .active:hover > a,
.bs-sidenav .nav > .active:focus > a {
   color: #5D6D7E;
   border-right: 0;
}
